"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.launchServer = void 0;
const cp = require("child_process");
/**
 * Forks a new server process.  By default, the server will not have ATA or produce diagnostic events.
 */
function launchServer(tsserverPath, args, execArgv) {
    var _a;
    const eventListeners = [];
    const exitListeners = [];
    const serverProc = cp.fork(tsserverPath, args !== null && args !== void 0 ? args : ["--disableAutomaticTypingAcquisition"], {
        execArgv: execArgv !== null && execArgv !== void 0 ? execArgv : (_a = process.execArgv) === null || _a === void 0 ? void 0 : _a.map(arg => bumpDebugPort(arg)),
        stdio: ["pipe", "pipe", "ignore", "ipc"]
    });
    const useNodeIpc = !!args && !!args.filter(a => a.toLocaleLowerCase() === "--useNodeIpc".toLocaleLowerCase()).length;
    const getNext = makeListeners(serverProc, useNodeIpc, eventListeners);
    serverProc.on("exit", code => {
        for (const listener of exitListeners) {
            listener(code);
        }
    });
    function on(event, listener) {
        switch (event) {
            case "event":
                eventListeners.push(listener);
                break;
            case "exit":
                exitListeners.push(listener);
                break;
        }
    }
    return {
        message: request => message(serverProc, useNodeIpc, getNext, request),
        kill: () => serverProc.kill(),
        on,
    };
}
exports.launchServer = launchServer;
function bumpDebugPort(arg) {
    const match = /^(--inspect(?:-brk)?)(?:=(\d+))?$/.exec(arg);
    return match
        ? `${match[1]}=${match[2] ? (+match[2] + 1) : 9230}`
        : arg;
}
function makeListeners(serverProc, useNodeIpc, eventListeners) {
    const waiters = new Map();
    const objects = new Map();
    if (useNodeIpc) {
        serverProc.on('message', handleMessage);
    }
    else {
        let unconsumedText = "";
        let currentLength = -1;
        serverProc.stdout.on('data', buffer => {
            unconsumedText += buffer.toString("utf8");
            while (true) {
                if (currentLength < 0) {
                    const headerMatch = unconsumedText.match(/Content-Length: (\d+)/);
                    if (!headerMatch)
                        break;
                    currentLength = +headerMatch[1];
                    unconsumedText = unconsumedText.substring(unconsumedText.indexOf("{", headerMatch.index + headerMatch.length));
                }
                if (unconsumedText.length < currentLength)
                    return;
                const obj = JSON.parse(unconsumedText.substring(0, currentLength));
                unconsumedText = unconsumedText.substring(currentLength);
                currentLength = -1;
                handleMessage(obj);
            }
        });
    }
    function handleMessage(obj) {
        if (obj.type === "event" && obj.event !== "requestCompleted") {
            for (const listener of eventListeners) {
                listener(obj);
            }
            return;
        }
        const requestSeq = obj.type === "event"
            ? obj.body.request_seq
            : obj.request_seq;
        const w = waiters.get(requestSeq);
        if (w) {
            waiters.delete(requestSeq);
            w(obj);
        }
        else {
            objects.set(requestSeq, obj);
        }
    }
    const getResponse = (seq) => new Promise(resolve => {
        const obj = objects.get(seq);
        if (obj) {
            objects.delete(seq);
            resolve(obj);
        }
        else {
            waiters.set(seq, resolve);
        }
    });
    return getResponse;
}
async function message(serverProc, useNodeIpc, getResponse, request) {
    const seq = request.seq;
    if (useNodeIpc) {
        serverProc.send(request);
    }
    else {
        serverProc.stdin.write(JSON.stringify(request) + "\n");
    }
    return await getResponse(seq);
}
//# sourceMappingURL=index.js.map